@@ -7,7 +7,7 @@ import logging
 from tornado import web
 from tornado import gen
 
-from litedfs.name.handlers.base import BaseHandler, BaseSocketHandler
+from litedfs.name.handlers.base import BaseHandler, BaseSocketHandler, auth_check
 from litedfs.name.models.data_nodes import DataNodes
 from litedfs.name.utils.listener import Connection
 from litedfs.name.utils.common import Errors, list_sort
@@ -26,6 +26,7 @@ class AboutHandler(BaseHandler):
 
 
 class ClusterInfoHandler(BaseHandler):
+    @auth_check
     @gen.coroutine
     def get(self):
         result = {"result": Errors.OK, "version": __version__}
