@@ -92,27 +92,28 @@ def download_directory(dir, remote_path, local_path, handler):
     target_path = os.path.join(local_path, dir["name"])
     if not os.path.exists(target_path):
         os.mkdir(target_path)
-    dirs, files = handler.client.listdir(source_path, sort_by = "name", desc = False)
-    for d in dirs:
-        download_directory(d, source_path, target_path, handler)
-    for f in files:
-        msg = {"cmd": "info"}
-        try:
-            source_file_path = os.path.join(source_path, f["name"])
-            target_file_path = os.path.join(target_path, f["name"])
-            if handler.client.download_file(source_file_path, target_file_path):
-                msg["info"] = "Download remote file [%s] to [%s] success" % (source_file_path, target_file_path)
-                LOG.info("Download remote file [%s] to [%s] success", source_file_path, target_file_path)
-            else:
+    items, _ = handler.client.listdir(source_path, sort_by = "name", desc = False)
+    for item in items:
+        if item["type"] == "Directory":
+            download_directory(item, source_path, target_path, handler)
+        else:
+            msg = {"cmd": "info"}
+            try:
+                source_file_path = os.path.join(source_path, item["name"])
+                target_file_path = os.path.join(target_path, item["name"])
+                if handler.client.download_file(source_file_path, target_file_path):
+                    msg["info"] = "Download remote file [%s] to [%s] success" % (source_file_path, target_file_path)
+                    LOG.info("Download remote file [%s] to [%s] success", source_file_path, target_file_path)
+                else:
+                    msg["cmd"] = "error"
+                    msg["info"] = "Download remote file [%s] to [%s] failed" % (source_file_path, target_file_path)
+                    LOG.info("Download remote file [%s] to [%s] failed", source_file_path, target_file_path)
+                time.sleep(0.1)
+            except Exception as e:
+                LOG.exception(e)
                 msg["cmd"] = "error"
-                msg["info"] = "Download remote file [%s] to [%s] failed" % (source_file_path, target_file_path)
-                LOG.info("Download remote file [%s] to [%s] failed", source_file_path, target_file_path)
-            time.sleep(0.1)
-        except Exception as e:
-            LOG.exception(e)
-            msg["cmd"] = "error"
-            msg["info"] = str(e)
-        MessageQueue.put([handler, msg])
+                msg["info"] = str(e)
+            MessageQueue.put([handler, msg])
     msg = {"cmd": "info"}
     msg["info"] = "Download remote directory [%s] to [%s] success" % (source_path, target_path)
     LOG.info("Download remote directory [%s] to [%s] success", source_path, target_path)
@@ -123,27 +124,28 @@ def upload_directory(dir, local_path, remote_path, handler, replica = 1):
     source_path = os.path.join(local_path, dir["name"])
     target_path = os.path.join(remote_path, dir["name"])
     handler.client.mkdir(target_path)
-    dirs, files = listdir(source_path)
-    for d in dirs:
-        upload_directory(d, source_path, target_path, handler, replica = replica)
-    for f in files:
-        msg = {"cmd": "info"}
-        try:
-            source_file_path = os.path.join(source_path, f["name"])
-            target_file_path = os.path.join(target_path, f["name"])
-            if handler.client.upload_file(source_file_path, target_file_path, replica = replica):
-                msg["info"] = "Upload file [%s] to [%s] success" % (source_file_path, target_file_path)
-                LOG.info("Upload file [%s] to [%s] success", source_file_path, target_file_path)
-            else:
+    items, _ = listdir(source_path)
+    for item in items:
+        if item["type"] == "Directory":
+            upload_directory(item, source_path, target_path, handler, replica = replica)
+        else:
+            msg = {"cmd": "info"}
+            try:
+                source_file_path = os.path.join(source_path, item["name"])
+                target_file_path = os.path.join(target_path, item["name"])
+                if handler.client.upload_file(source_file_path, target_file_path, replica = replica):
+                    msg["info"] = "Upload file [%s] to [%s] success" % (source_file_path, target_file_path)
+                    LOG.info("Upload file [%s] to [%s] success", source_file_path, target_file_path)
+                else:
+                    msg["cmd"] = "error"
+                    msg["info"] = "Upload file [%s] to [%s] failed" % (source_file_path, target_file_path)
+                    LOG.info("Upload file [%s] to [%s] failed", source_file_path, target_file_path)
+                time.sleep(0.1)
+            except Exception as e:
+                LOG.exception(e)
                 msg["cmd"] = "error"
-                msg["info"] = "Upload file [%s] to [%s] failed" % (source_file_path, target_file_path)
-                LOG.info("Upload file [%s] to [%s] failed", source_file_path, target_file_path)
-            time.sleep(0.1)
-        except Exception as e:
-            LOG.exception(e)
-            msg["cmd"] = "error"
-            msg["info"] = str(e)
-        MessageQueue.put([handler, msg])
+                msg["info"] = str(e)
+            MessageQueue.put([handler, msg])
     msg = {"cmd": "info"}
     msg["info"] = "Upload directory [%s] to [%s] success" % (source_path, target_path)
     LOG.info("Upload directory [%s] to [%s] success", source_path, target_path)
@@ -152,26 +154,27 @@ def upload_directory(dir, local_path, remote_path, handler, replica = 1):
 
 def update_directory(dir, remote_path, handler, replica):
     dir_path = os.path.join(remote_path, dir["name"])
-    dirs, files = handler.client.listdir(dir_path, sort_by = "name", desc = False)
-    for d in dirs:
-        update_directory(d, dir_path, handler, replica = replica)
-    for f in files:
-        msg = {"cmd": "info"}
-        try:
-            f_path = os.path.join(dir_path, f["name"])
-            if handler.client.update_file(f_path, replica = replica):
-                msg["info"] = "Update remote file [%s] success" % f_path
-                LOG.info("Update remote file [%s] success", f_path)
-            else:
+    items, _ = handler.client.listdir(dir_path, sort_by = "name", desc = False)
+    for item in items:
+        if item["type"] == "Directory":
+            update_directory(item, dir_path, handler, replica = replica)
+        else:
+            msg = {"cmd": "info"}
+            try:
+                f_path = os.path.join(dir_path, item["name"])
+                if handler.client.update_file(f_path, replica = replica):
+                    msg["info"] = "Update remote file [%s] success" % f_path
+                    LOG.info("Update remote file [%s] success", f_path)
+                else:
+                    msg["cmd"] = "error"
+                    msg["info"] = "Update remote file [%s] failed" % f_path
+                    LOG.info("Update remote file [%s] failed", f_path)
+                time.sleep(0.1)
+            except Exception as e:
+                LOG.exception(e)
                 msg["cmd"] = "error"
-                msg["info"] = "Update remote file [%s] failed" % f_path
-                LOG.info("Update remote file [%s] failed", f_path)
-            time.sleep(0.1)
-        except Exception as e:
-            LOG.exception(e)
-            msg["cmd"] = "error"
-            msg["info"] = str(e)
-        MessageQueue.put([handler, msg])
+                msg["info"] = str(e)
+            MessageQueue.put([handler, msg])
     msg = {"cmd": "info"}
     msg["info"] = "Update remote directory [%s] success" % dir_path
     LOG.info("Update remote directory [%s] success", dir_path)
@@ -449,6 +452,36 @@ class TaskProcesser(StoppableThread):
                                         MessageQueue.put([task["socket_handler"], msg])
                                     msg = {"cmd": Command.need_refresh, "dir_path": dir_path}
                                     MessageQueue.put(["remote", msg])
+                                elif task["cmd"] == Command.preview:
+                                    dir_path = joinpath(task["dir_path"])
+                                    file = task["file"]
+                                    file_type = file["type"].lower()
+                                    file_path = os.path.join(dir_path, file["name"])
+                                    msg = {}
+                                    if file_type == ".zip":
+                                        namelist = task["socket_handler"].client.preview_zip_file(file_path)
+                                        if namelist:
+                                            msg["cmd"] = Command.preview
+                                            msg["file_path"] = file_path
+                                            msg["data"] = namelist
+                                            msg["type"] = file_type
+                                        else:
+                                            msg["cmd"] = "error"
+                                            msg["info"] = "preview file [%s] failed" % file_path
+                                    elif file["name"].lower() == ".gitignore" or file_type in [".txt", ".log", ".yml", ".json", ".md", ".py", ".c", ".xml", ".go", ".sh", ".html"]:
+                                        content = task["socket_handler"].client.preview_text_file(file_path)
+                                        if content:
+                                            msg["cmd"] = Command.preview
+                                            msg["file_path"] = file_path
+                                            msg["data"] = content
+                                            msg["type"] = file_type
+                                        else:
+                                            msg["cmd"] = "error"
+                                            msg["info"] = "preview file [%s] failed" % file_path
+                                    else:
+                                        msg["cmd"] = "error"
+                                        msg["info"] = "preview does not support this type of file"
+                                    MessageQueue.put([task["socket_handler"], msg])
                             else:
                                 time.sleep(0.5)
                             LOG.info("TaskProcesser(%03d) process task: %s", self.pid, task)
